import pendulum
from marshmallow import Schema, fields, post_load
from workspaces.schema import WorkflowEventType, WorkflowEvent, CapabilityEvent


class WorkflowEventSchema(Schema):
    """
    Default schema for serializing WorkflowEvents.
    """
    job_name = fields.String()
    job_id = fields.String()
    event_type = fields.Method("get_type", deserialize="load_type")
    timestamp = fields.String()
    log = fields.String()
    retval = fields.Integer(allow_none=True)

    # Enums apparently are not a first-class field type in Marshmallow
    def get_type(self, obj: WorkflowEventType) -> str:
        return obj.type.name.lower()

    def load_type(self, name: str) -> WorkflowEventType:
        return next(et for et in WorkflowEventType if et.name.lower() == name)

    @post_load
    def make_event(self, data, **kwargs):
        return WorkflowEvent(**data)


class CapabilityEventSchema(Schema):
    """
    Default schema for serializing CapabilityEvents.
    """

    def get_type(self) -> str:
        raise NotImplementedError

    def load_type(self, name:str):
        raise NotImplementedError

    @post_load
    def make_event(self, data, **kwargs):
        return CapabilityEvent(**data)


workflow_event = WorkflowEventSchema()
capability_event = CapabilityEventSchema()
